#region usings
using System;
using System.ComponentModel.Composition;
using System.Runtime.InteropServices;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{	
	[StructLayout(LayoutKind.Sequential)]
	public struct MyParticle : ILocatable, IScalable
	{
		public Vector3D Position {get; set;}
		private Vector3D FLastPosition;
		public Vector3D LastPosition 
		{
			get 
			{
				return FFirstFrame ? Position : FLastPosition;
			}
		}
		public Vector3D LastVelocity {get; set;}
		public Vector3D Scaling {get; set;}
		
		private bool FFirstFrame;
		
		public MyParticle Init()
		{
			FFirstFrame = true;
			Scaling = new Vector3D(1, 1, 1);
			LastVelocity = new Vector3D(0, 0, 0);
			return this;
		}	
		
		public void Step()
		{
			LastVelocity = Position-LastPosition;
			FLastPosition = Position;
			FFirstFrame = false;
		}	
	}
			
	[PluginInfo(Name = "Create", Category = "MyParticle", Version = "", Help = "create my own particles", Tags = "")]
	public class MyParticleCreateNode : IPluginEvaluate
	{
		[Input("Count", IsSingle = true, DefaultValue = 1)]		
		ISpread<int> FCount;
		
		[Output("Output")]		ISpread<MyParticle> FOutput;
		
		public void Evaluate(int SpreadMax)
		{
			SpreadMax = FCount[0];
			FOutput.SliceCount = SpreadMax;

			for (int i = 0; i < SpreadMax; i++)
				FOutput[i] = new MyParticle().Init();
		}
	}
		
	[PluginInfo(Name = "SetPosition", Category = "MyParticle", Help = "Set new Position", Tags = "")]
	public class MyParticleSetPositionNode : ParticleSetPositionNode<MyParticle>
	{
	}	
	
	[PluginInfo(Name = "SetVelocity", Category = "MyParticle", Help = "Position = LastPosition + Velocity", Tags = "")]
	public class MyParticleSetVelocityNode : ParticleSetVelocityNode<MyParticle>
	{
	}	 
	
	[PluginInfo(Name = "Step", Category = "MyParticle", Help = "Acknowledge one step in the animation. Typically used once per Frame.", Tags = "")]
	public class MyParticleStepNode : ParticleStepNode<MyParticle>
	{
	}	
	  
	[PluginInfo(Name = "SetScaling", Category = "MyParticle", Help = "Set new Scaling", Tags = "")]
	public class MyParticleSetScalingNode : ParticleSetScalingNode<MyParticle>
	{
	}		
	
	[PluginInfo(Name = "Cons", Category = "MyParticle", Help = "", Tags = "")]
	public class MyParticleConsNode : FlatCons<MyParticle>
	{ 
	}	
	
	[PluginInfo(Name = "SetState", Category = "MyParticle", AutoEvaluate = true, Help = "", Tags = "")]
	public class MyParticleSetStateNode : ParticleSetStateNode<MyParticle>
	{
	}	 

	[PluginInfo(Name = "GetState", Category = "MyParticle", Help = "", Tags = "")]
	public class MyParticleGetStateNode : ParticleGetStateNode<MyParticle>
	{
	}	 	
}
